from typing import final
from gwpy.timeseries import TimeSeries
from h5py import File, Group, Dataset

from .section import Section
from .metadata.component import Component as Metadata
from .utils import dataset_to_timeseries, timeseries_to_dataset


@final
class Component(Section):
    """
    A Component is an element of the interferometer that is associated
    with a movement sensor. Right now, we only store movement in the z
    axis.
    Attributes
    ----------
    movement : gwpy.timeseries.TimeSeries[length]
        Corrected movement of the component
    """

    def __init__(self, metadata: Metadata, sensor: TimeSeries):
        """
        Parameters
        ----------
        metadata : vda.package.metadata.Component
            Metadata associated to this data
        sensor : gwpy.timeseries.TimeSeries[Volt]
            Sensor data of the component (in Volt, not in meter !)
        Warnings
        --------
        It's an internal class, of whose instances are not produced via
        a direct call, but by using helper functions
        See Also
        --------
        bed.MetadataComponent : Associated metadata type
        """
        self.metadata = metadata
        self.sensor = sensor
        self.movement = self.sensor * self.metadata.calibration_factor

    @staticmethod
    def from_hdf5(store: File | Group) -> "Component":
        sensor_dataset = store.get("sensor")
        if not isinstance(sensor_dataset, Dataset):
            raise ValueError(
                "sensor should exists, and not be {}".format(
                    type(sensor_dataset)
                )
            )
        sensor = dataset_to_timeseries(sensor_dataset)
        return Component(
            Metadata.from_hdf5(store),
            sensor=sensor,
        )

    def to_hdf5(self, store: File | Group) -> Group | File:
        _ = self.metadata.to_hdf5(store)
        _ = timeseries_to_dataset(store, "sensor", self.sensor)
        return store
